/* Customizations to the theme */

/* override table width restrictions */
/* https://github.com/readthedocs/sphinx_rtd_theme/issues/117 */
@media screen and (min-width: 768px) {
    .wy-table-responsive table td, .wy-table-responsive table th {
        white-space: normal !important;
    }

    .wy-table-responsive {
        overflow: visible !important;
        max-width: 100%;
    }
}

.wy-side-nav-search, .wy-nav-top {
    background: #444466;
}
